"""
CutoML - A lightweight automl framework for classification and regression tasks.

Copyright (C) 2022  Omkar Udawant

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

# Classifiers
from sklearn.naive_bayes import MultinomialNB
from sklearn.naive_bayes import GaussianNB
from sklearn.naive_bayes import BernoulliNB
from sklearn.linear_model import LogisticRegression
from sklearn.linear_model import SGDClassifier
from sklearn.svm import LinearSVC
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.ensemble import ExtraTreesClassifier
from xgboost import XGBClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.neural_network import MLPClassifier

# Regressors
from sklearn.linear_model import LinearRegression
from sklearn.linear_model import ElasticNet
from sklearn.linear_model import Ridge
from sklearn.linear_model import Lasso
from sklearn.linear_model import LassoLars
from sklearn.linear_model import SGDRegressor
from sklearn.ensemble import ExtraTreesRegressor
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import AdaBoostRegressor
from sklearn.svm import SVR, LinearSVR
from sklearn.tree import DecisionTreeRegressor
from sklearn.neighbors import KNeighborsRegressor
from xgboost import XGBRegressor
from sklearn.experimental import enable_halving_search_cv
from sklearn.model_selection import HalvingRandomSearchCV
import numpy as np


class Classifiers:
    def __init__(self,
                 k_folds=5,
                 n_jobs=2,
                 verbose=1,
                 random_state=0):
        self.models = [
            HalvingRandomSearchCV(
                estimator=GaussianNB(),
                param_distributions={
                    'var_smoothing': np.logspace(0, -9, 10)
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='f1_weighted',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=BernoulliNB(),
                param_distributions={
                    'alpha': [1e-3, 1e-2, 1e-1, 1., 10., 20.],
                    'fit_prior': [True, False]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='f1_weighted',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=MultinomialNB(),
                param_distributions={
                    'alpha': [1e-3, 1e-2, 1e-1, 1., 10., 20.],
                    'fit_prior': [True, False]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='f1_weighted',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                ExtraTreesClassifier(n_jobs=n_jobs,
                                     random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'criterion': ["gini", "entropy"],
                    'max_features': np.arange(0.05, 1.00, 0.05),
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                    'bootstrap': [True, False]
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                KNeighborsClassifier(n_jobs=n_jobs),
                param_distributions={
                    'n_neighbors': [2, 4, 8, 10, 12],
                    'weights': ["uniform", "distance"],
                    'p': [1, 2]
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                LogisticRegression(n_jobs=n_jobs,
                                   random_state=random_state),
                param_distributions={
                    'penalty': ["l1", "l2"],
                    'C': np.logspace(0, 3, 4),
                    'max_iter': [100, 500]
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                SGDClassifier(n_jobs=n_jobs,
                              random_state=random_state),
                param_distributions={
                    'loss': ['log', 'hinge', 'modified_huber', 'squared_hinge',
                             'perceptron'],
                    'penalty': ['elasticnet'],
                    'alpha': [0.0, 0.01, 0.001],
                    'learning_rate': ['invscaling', 'constant'],
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                LinearSVC(random_state=random_state),
                param_distributions={
                    'penalty': ["l2"],
                    'dual': [True],
                    'max_iter': [100, 500],
                    'tol': [1e-3, 1e-2, 1e-1],
                    'C': np.logspace(0, 3, 4),
                },
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                cv=k_folds,
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                LinearSVC(random_state=random_state),
                param_distributions={
                    'penalty': ["l1", "l2"],
                    'max_iter': [100, 500],
                    'dual': [False],
                    'tol': [1e-3, 1e-2, 1e-1],
                    'C': np.logspace(0, 3, 4)
                },
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                cv=k_folds,
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                RandomForestClassifier(
                    n_jobs=n_jobs,
                    random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'max_features': ['log2', 'sqrt'],
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                GradientBoostingClassifier(random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                XGBClassifier(n_jobs=n_jobs,
                              random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                    'min_child_weight': range(1, 21),
                    'subsample': np.arange(0.05, 1.0, 0.05),
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                DecisionTreeClassifier(random_state=random_state),
                param_distributions={
                    'criterion': ["gini", "entropy"],
                    'max_depth': range(1, 21),
                    'min_samples_split': range(2, 21),
                    'min_samples_leaf': range(1, 21)
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                scoring='f1_weighted',
                n_jobs=n_jobs
            ),
            HalvingRandomSearchCV(
                MLPClassifier(random_state=random_state),
                param_distributions={
                    'max_iter': [100, 500],
                    'hidden_layer_sizes': [
                        (10, 30, 10),
                        (50, 50, 50),
                        (50, 100, 50),
                        (20,),
                        (100,)
                    ],
                    'activation': ['tanh', 'relu'],
                    'solver': ['sgd', 'adam'],
                    'alpha': [0.0001, 0.05],
                    'learning_rate': ['constant', 'adaptive'],
                },
                cv=k_folds,
                random_state=random_state,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='f1_weighted',
            )
        ]


class Regressors:
    def __init__(self,
                 k_folds=5,
                 n_jobs=2,
                 verbose=1,
                 random_state=0):
        self.models = [
            LinearRegression(n_jobs=n_jobs),
            HalvingRandomSearchCV(
                estimator=ElasticNet(random_state=random_state),
                param_distributions={
                    'l1_ratio': np.arange(0.0, 1.0, 0.05),
                    'tol': [1e-3, 1e-2, 1e-1]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=ExtraTreesRegressor(random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'criterion': ["gini", "entropy"],
                    'max_features': np.arange(0.05, 1.00, 0.05),
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                    'bootstrap': [True, False]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=GradientBoostingRegressor(random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'loss': ["ls", "lad", "huber", "quantile"],
                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                    'max_features': ['log2', 'sqrt'],
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                    'alpha': [0.75, 0.8, 0.85, 0.9, 0.95, 0.99]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=AdaBoostRegressor(random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                    'loss': ["linear", "square", "exponential"]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=DecisionTreeRegressor(random_state=random_state),
                param_distributions={
                    'max_depth': range(1, 11),
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=KNeighborsRegressor(n_jobs=n_jobs),
                param_distributions={
                    'n_neighbors': [2, 4, 8, 10, 12],
                    'p': [1, 2]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=Ridge(random_state=random_state),
                param_distributions={
                    'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=Lasso(random_state=random_state),
                param_distributions={
                    'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=LassoLars(random_state=random_state),
                param_distributions={
                    'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=LinearSVR(random_state=random_state),
                param_distributions={
                    'loss': ["epsilon_insensitive",
                             "squared_epsilon_insensitive"],
                    'dual': [True],
                    'tol': [1e-3, 1e-2, 1e-1],
                    'C': np.logspace(0, 3, 4),
                    'epsilon': [1e-3, 1e-2, 1e-1, 1.]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=LinearSVR(random_state=random_state),
                param_distributions={
                    'loss': ["squared_epsilon_insensitive"],
                    'dual': [False],
                    'tol': [1e-3, 1e-2, 1e-1],
                    'C': np.logspace(0, 3, 4),
                    'epsilon': [1e-3, 1e-2, 1e-1, 1.]
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=RandomForestRegressor(
                    n_jobs=n_jobs,
                    random_state=random_state),
                param_distributions={
                    'max_iter': [100],
                    'max_features': ['log2', 'sqrt', ],
                    'min_samples_split': [2, 5, 10, 15, 20],
                    'min_samples_leaf': [1, 2, 5, 10],
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=XGBRegressor(
                    n_jobs=n_jobs,
                    random_state=random_state),
                param_distributions={
                    'n_estimators': [100],
                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                    'min_child_weight': range(1, 21),
                    'subsample': np.arange(0.05, 1.0, 0.05),
                    'min_child_weight': range(1, 21),
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            ),
            HalvingRandomSearchCV(
                estimator=SGDRegressor(random_state=random_state),
                param_distributions={
                    'loss': ['squared_loss', 'huber', 'epsilon_insensitive'],
                    'penalty': ['elasticnet'],
                    'alpha': [0.0, 0.01, 0.001],
                    'learning_rate': ['invscaling', 'constant'],
                },
                cv=k_folds,
                verbose=verbose,
                n_jobs=n_jobs,
                scoring='r2',
                random_state=random_state
            )
        ]



"""
CutoML - A lightweight automl framework for classification and regression tasks.

Copyright (C) 2022 Omkar Udawant

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import tqdm
from imblearn.over_sampling import SMOTE
from pathos.multiprocessing import ProcessingPool
from sklearn.model_selection import train_test_split
from sklearn.pipeline import Pipeline

from cutoml.config import Classifiers
from cutoml.config import Regressors
from cutoml.utils import timer
import time
import warnings

warnings.filterwarnings('ignore')


class CutoClassifier:
    def __init__(self, k_folds=3, oversample=False, n_jobs=1, verbose=0, random_state=0):
        self.random_state = random_state
        classifiers_config = Classifiers(
            k_folds=k_folds,
            n_jobs=n_jobs,
            verbose=verbose,
            random_state=self.random_state
        )
        self.models = classifiers_config.models
        self.oversample = oversample
        self.best_estimator = None
        self.n_jobs = n_jobs

    def _model_fitter(self, model, X, y):
        try:
            clf = Pipeline([
                ('classification_model', model)
            ])
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore")
                clf = clf.fit(X, y)
            return clf
        except Exception as e:
            pass

    def fit(self, X, y):
        X_train, X_test, y_train, y_test = train_test_split(
            X, y, test_size=0.2, random_state=self.random_state
        )

        if self.oversample:
            over_sampler = SMOTE(
                random_state=self.random_state,
                k_neighbors=3,
                n_jobs=self.n_jobs
            )
            X_train, y_train = over_sampler.fit_resample(
                X_train, y_train
            )

        start_time = time.time()

        pool = ProcessingPool()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            try:
                *trained_pipelines, = tqdm.tqdm(
                    pool.imap(
                        lambda x: self._model_fitter(x,
                                                     X_train,
                                                     y_train),
                        self.models,
                        chunksize=len(self.models) // 2
                    ),
                    total=len(self.models),
                    desc='Optimizing'
                )

                end_time = time.time()
            except KeyboardInterrupt:
                pool.terminate()
                pool.join()
            finally:
                pool.close()
                pool.join()
        print(timer(start=start_time, end=end_time))
        trained_models = dict()
        for pipeline in trained_pipelines:
            if pipeline:
                try:
                    score = pipeline.score(X_test, y_test)
                    trained_models[score] = pipeline
                except Exception as e:
                    pass
        if trained_models:
            self.best_estimator = max(
                sorted(trained_models.items(), reverse=True))[1]
            return self
        else:
            raise RuntimeError('Could not find best estimator.')

    def predict(self, X):
        if not self.best_estimator:
            raise RuntimeError(
                'Models not fit yet, please call fit() method first.')
        prediction = self.best_estimator.predict(X)
        return prediction

    def predict_proba(self, X):
        if not self.best_estimator:
            raise RuntimeError(
                'Models not fit yet, please call fit() method first.')
        prediction_probablity = self.best_estimator.predict_proba(X)
        return prediction_probablity

    def score(self, X, y):
        if not self.best_estimator:
            raise RuntimeError(
                'Models not fit yet, please call fit() method first.')
        score = self.best_estimator.score(X, y)
        return score


class CutoRegressor:
    def __init__(self, k_folds=3, n_jobs=1, verbose=0, random_state=0):
        self.random_state = random_state
        regressors_config = Regressors(
            k_folds=k_folds,
            n_jobs=n_jobs,
            verbose=verbose,
            random_state=self.random_state
        )
        self.models = regressors_config.models
        self.best_estimator = None
        self.n_jobs = n_jobs

    def _model_fitter(self, model, X, y):
        try:
            rgr = Pipeline([
                ('regression_model', model)
            ])
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                rgr = rgr.fit(X, y)
            return rgr
        except Exception as e:
            print(e)
            pass

    def fit(self, X, y):
        X_train, X_test, y_train, y_test = train_test_split(
            X, y, test_size=0.2, random_state=self.random_state
        )
        start_time = time.time()
        pool = ProcessingPool()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            try:
                *trained_pipelines, = tqdm.tqdm(
                    pool.imap(
                        lambda x: self._model_fitter(x,
                                                     X_train,
                                                     y_train),
                        self.models,
                        chunksize=len(self.models) // 2
                    ),
                    total=len(self.models),
                    desc='Optimizing'
                )

                end_time = time.time()
            except KeyboardInterrupt:
                pool.terminate()
                pool.join()
            finally:
                pool.close()
                pool.join()
        print(timer(start=start_time, end=end_time))
        trained_models = dict()
        for pipeline in trained_pipelines:
            if pipeline:
                try:
                    score = pipeline.score(X_test, y_test)
                    trained_models[score] = pipeline
                except Exception as e:
                    pass
        if trained_models:
            self.best_estimator = max(
                sorted(trained_models.items(), reverse=True))[1]
            return self
        else:
            raise RuntimeError('Could not find best estimator.')

    def predict(self, X):
        if not self.best_estimator:
            raise RuntimeError(
                'Models not fit yet, please call fit() method first.')
        prediction = self.best_estimator.predict(X)
        return prediction

    def score(self, X, y):
        if not self.best_estimator:
            raise RuntimeError(
                'Models not fit yet, please call fit() method first.')
        score = self.best_estimator.score(X, y)
        return score



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='cutoml',
    packages=['cutoml'],
    version='0.0.10',
    license='gpl-3.0',
    description='A lightweight automl library',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Omkar Udawant',
    author_email='omkarudawant97@gmail.com',
    url='https://github.com/omkarudawant/CutoML',
    download_url='https://github.com/omkarudawant/CutoML/archive/refs/tags/0.0.10.tar.gz',
    keywords=[
        'pipeline optimization',
        'automated hyperparameter optimization',
        'data science',
        'machine learning',
    ],
    install_requires=[
        'scipy>=1.5.1',
        'numpy>=1.19.1',
        'scikit-learn>=0.24.0',
        'xgboost<1.3.0',
        'tqdm>=4.56.0',
        'pathos>=0.2.7',
        'imbalanced-learn>=0.8.0',
        'setuptools==58.0.3'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'License :: OSI Approved :: GNU Lesser General Public License v3 ('
        'LGPLv3)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    python_requires='>=3.6',
)



"""
CutoML - A lightweight automl framework for classification and regression tasks.

Copyright (C) 2022  Omkar Udawant

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from sklearn.metrics import f1_score
from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import roc_auc_score
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error
from sklearn.metrics import r2_score
from sklearn.preprocessing import LabelBinarizer
import numpy as np
import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter("ignore")


def multi_class_roc_auc_score(y_true: np.array, y_pred: np.array,
                              average='weighted'):
    lb = LabelBinarizer()
    lb.fit(y_true)
    y_true = lb.transform(y_true)
    y_pred = lb.transform(y_pred)
    return roc_auc_score(y_true, y_pred, average=average)


def mean_absolute_percentage_error(y_true, y_pred):
    mask = y_true != 0
    return (np.fabs(y_true - y_pred) / y_true)[mask].mean()


def classification_metrics(y_true: np.array, y_pred: np.array):
    average = 'weighted' if len(np.unique(y_true)) > 2 else None
    f1 = f1_score(y_true=y_true, y_pred=y_pred, average=average)
    precision = precision_score(y_true=y_true, y_pred=y_pred, average=average)
    recall = recall_score(y_true=y_true, y_pred=y_pred, average=average)
    accuracy = accuracy_score(y_true=y_true, y_pred=y_pred)
    roc_auc_ = multi_class_roc_auc_score(y_true=y_true, y_pred=y_pred,
                                         average=average)
    return accuracy, f1, precision, recall, roc_auc_


def regression_metrics(y_true: np.array, y_pred: np.array):
    mse = mean_squared_error(y_true=y_true, y_pred=y_pred)
    mae = mean_absolute_error(y_true=y_true, y_pred=y_pred)
    mape = mean_absolute_percentage_error(y_true=y_true, y_pred=y_pred)
    r2 = r2_score(y_true=y_true, y_pred=y_pred)
    return r2, mape, mse, mae


def timer(start, end):
    hours, rem = divmod(end - start, 3600)
    minutes, seconds = divmod(rem, 60)
    return "Time elapsed: {:0>2}:{:0>2}:{:05.2f} hh:mm:ss".format(
        int(hours),
        int(minutes),
        seconds)
